
##############################################################################
#
# srvconf.tcl --
#
#    Server startup and configuration procedure. This procedure is
#    executed at server startup only. Therefore only limited server 
#    command-set is available.
#
#    It should not be necessary to modify this file. Additional server
#    parameters can be set and/or overriden in file located in the 
#    config/<server_name>.<server_port> per-server configuration file.
#
#    See the file "license.txt" for information on usage and
#    redistribution of this file, and for a DISCLAIMER OF ALL WARRANTIES.
#
# Rcsid: @(#)$Id: vsrvconf.tcl,v 1.5 2013/03/19 14:19:33 zv Exp $
#
##############################################################################

#
# These two variables configure the control-port access information.
# You can declare only one control-port user. 
# You may choose to set this file to be viewed by the system privileged
# user (root) in order to keep the control-port user password secret.
#

set admin_user "nsadmin"
set admin_pass "x"

#
# Generic server configuration procedure. Generally, you should not
# change anything below, unless instructed by Archiware tech-support.
#

proc srvconf {args} {

    global server hostname vservers admin_user admin_pass

    #
    # MIME types. You can extend this section by adding additional
    # "ns_param" lines giving link between the file extension and
    # the corresponding mime type, like: ns_param ".gif" "image/gif".
    # Server has a reasonable number of common types already built-in.
    #

    ns_section ns/mimetypes

    ns_param noextension "*/*"
    ns_param default     "*/*"

    #
    # Parse defined virtual servers and matching domains
    #

    set vsnames "" ; # Names of virtual servers
    set vshosts "" ; # Domains or IP addresses

    if {[info exists vservers] && $vservers != ""} {
        foreach {vs ip} $vservers {
            lappend vsnames $vs
            lappend vshosts $ip
        }
    }

    #
    # This is the name of the current configuration file. 
    # We assume several things about it's name and location.
    # The current configuration file must be:
    #
    #   - located under <server_home>/config/ directory
    #   - named like <server_name>.<http_port>
    #

    set cf [ns_info config]

    #
    # Change to directory where the configuration file is stored.
    # We must do this in order to obtain it's absolute path.
    #
    
    set cwd [pwd]; cd [file dirname $cf]
    set cfd [pwd]; cd $cwd
    
    #
    # Assuming above, server's home must be one directory
    # level above current configuration file directory.
    #

    set home [file dirname $cfd]

    #
    # With little arithmetic, we obtain the server name 
    # and it's default http port.
    #

    set server   [lindex [split [file tail $cf] "."] 0]
    set httpport [lindex [split [file tail $cf] "."] 1]
 
    #
    # Test directory structure and create it, if needed.
    #

    foreach vs [concat $server $vsnames] {
        foreach dir {
            pages modules modules/tcl modules/nslog modules/nsperm
        } {
            file mkdir $home/servers/$vs/$dir
        }
    }

    #
    # Global server parameters. There are myriad of server-related
    # parameters which one can put here. We use defaults for most,
    # and explicitly set only very few of them.
    #

    ns_section ns/parameters

    ns_param home      $home
    ns_param pidfile   log/$server.pid
    ns_param logroll   false 
    ns_param serverlog $server.log

    #
    # Rotate server log file per hand since there
    # is no support for this in the core server.
    #

    logroll $home/log/$server.log
    
    if {[file exists $home/log/$server.log]} {
        catch {file rename $home/log/$server.log $home/log/$server.log.0}
    }

    #
    # Empirically determined minimum thread stacksize.
    #

    ns_section ns/threads
    ns_param stacksize [expr 256 * 1024]

    #
    # Socket module parameters. This module
    # is global to all virtual servers.
    #

    ns_section ns/module/nssock

    if {![info exists httpport]} {
        set httpport 80
    } else {
        ns_param port $httpport
    }
    if {![info exists hostname]} {
        set hostname [ns_info hostname]
    } else {
        ns_param hostname [lindex $hostname 0]
    }

    #
    # Sort-out binary modules and distribute them
    # among virtual servers. This is done for
    # compatibility reasons with previous versions.
    #

    switch -glob -- $::tcl_platform(os) {
        Darwin {
            set darwin 1
            set shlibs [list *.so *.dylib]
        }
        Windows* {
            set darwin 0
            set shlibs [list *.dll]
        } 
        default {
            set darwin 0
            set shlibs [list *.so]
        }
    }

    foreach ext $shlibs {
        foreach file [lsort [glob -nocomp $home/bin/$ext]] {
            set lib [file tail $file]
            set mod [file root $lib]
            switch -glob $lib {
                libns*      -
                libtcl*     -
                nsopenssl*  -
                gdbm.dll    -
                nsd.dll     -
                nstcl.dll   -
                tcl84tg.dll -
                msvcrtd.dll -
                nsthread.dll {
                    continue ; # Ignore: not a module
                }
                nssock* {
                    lappend glob_mods $mod $file
                }
                nslog* - nscp* - nscgi* {
                    lappend virt_mods $mod $file
                }
                libdisktool* {
                    if {$darwin && $::tcl_platform(osVersion) >= {7.0.0}} {
                        lappend main_mods $mod $file
                    }
                }
                default {
                    lappend main_mods $mod $file
                }
            }
        }
    }

    #
    # Load server-wide binary modules.
    #

    ns_section ns/modules
    foreach {mod lib} $glob_mods {
        ns_param $mod $lib
    }

    #
    # Load main-server-only binary modules.
    #

    ns_section ns/server/$server/modules
    foreach {mod lib} $main_mods {
        ns_param $mod $lib
    }    
    
    #
    # Load virtual server binary modules.
    #

    foreach vs [concat $server $vsnames] {
        ns_section ns/server/$vs/modules
        foreach {mod lib} $virt_mods {
            ns_param $mod $lib
        }
    }

    #
    # Register virtual servers.
    #

    ns_section ns/servers
    foreach vs [concat $server $vsnames] {
        ns_param $vs $vs
    }

    #
    # Map the main server to listen on all interfaces.
    #

    ns_section ns/module/nssock/servers
    foreach addr "[ns_info address] localhost 127.0.0.1 $hostname" {
        ns_param $server $addr
        ns_param $server $addr:$httpport
    }

    #
    # Map virtual servers to their respective domains.
    # We allow only one domain per server per default.
    #

    ns_section ns/module/nssock/servers
    foreach vs $vsnames addr $vshosts {
        ns_param $vs $addr
        ns_param $vs $addr:$httpport
    }

    #
    # Virtual server parameters. There are myriad of server-related
    # parameters which one can put here. We use defaults for most.
    #

    set vscount -1

    foreach vs [concat $server $vsnames] {
        
        #
        # Setup basic virtual server parameters.
        # 
    
        ns_section ns/server/$vs
        ns_param directoryfile index.adp,index.html,index.htm
        ns_param pageroot $home/servers/$vs/pages

        #
        # Control port parameters. Each virtual server gets
        # a control port with the shift of 1000 plus it's
        # order number * 10. The main server gets order of zero.
        #
        
        ns_section ns/server/$vs/module/nscp
        ns_param address 127.0.0.1
        ns_param port [expr {$httpport + 1000 + [incr vscount]*10}]

        ns_section ns/server/$vs/module/nscp/users
        ns_param user $admin_user:[ns_crypt $admin_pass ab]:

        #
        # Define per-server Tcl bootstrap files.
        #

        ns_section ns/server/$vs/tcl
        if {$vs == $server} {
            set initfile [file join [ns_info home] lib init.tcl]
        } else {
            set initfile [file join [ns_info home] lib $vs.tcl]
        }
        if {[file exists $initfile]} {
            ns_param initfile $initfile
        }

        #
        # Load Tcl-only modules, shared and private.
        #
        
        ns_section ns/server/$vs/modules
        foreach file [lsort [glob -nocomp $home/modules/tcl/*]] {
            if {[file isdirectory $file]} {
                ns_param [file tail $file] tcl
            }
        }
        foreach file [lsort [glob -nocompl $home/servers/$vs/modules/tcl/*]] {
            if {[file isdirectory $file]} {
                ns_param [file tail $file] tcl
            }
        }

        #
        # Adjust default cache parameters for fastpath module.
        # Notice that same parameters are used for nsxcmd_cache.
        #
        
        ns_section ns/server/$vs/fastpath
        ns_param mmap  true
        ns_param cache true
        ns_param cachemaxentry [expr {8*1024}]
        
        #
        # Access log parameters.
        #
        
        ns_section ns/server/$vs/module/nslog
        ns_param maxbackup    7
        ns_param rollhour     0
        ns_param rolllog      true
        ns_param rollonsignal true

        #
        # ADP mappings.
        #

        ns_section ns/server/$vs/adp
        ns_param map /*.adp
    }
}

proc logroll {path {history 100}} {

    # @c Rotates files: (path).(a) -> (path).(a+1); (a >= 0).
    # @c The file (path) is left intact. User has to rename it himself.
    # @a path: Path of the file to rotate
    # @a history: Number of history files to keep (default = 10)

    #
    # Collect all (path).(a) files.
    #

    set res [list]

    foreach name [glob -nocomplain $path.*] {
        set index [string trimleft $name $path.]
        if {[string is integer -strict $index]} {
            lappend res $index
        }
    }

    #
    # Roll'em over.
    #

    foreach index [lsort -integer -decreasing $res] {
        if {$index < $history} {
            catch {file rename -force $path.$index $path.[incr index]}
        }
    }
}

#
# Now, run the configurator procedure
#

srvconf

# EOF $RCSfile: vsrvconf.tcl,v $
